from wsgiref.simple_server import make_server
from pyramid.config import Configurator
from pyramid.response import Response
from pycapo import CapoConfig
import smtplib
import chevron
import zope.sqlalchemy

from workspaces.schema import get_engine, get_session_factory
from workspaces.services import NotificationInfo

def get_tm_session(session_factory, transaction_manager):
    """
    Enable Zope's transaction manager on our session
    :param session_factory:
    :param transaction_manager:
    :return:
    """
    dbsession = session_factory()
    zope.sqlalchemy.register(dbsession, transaction_manager=transaction_manager)
    return dbsession


def send_email(request):
    json_packet = request.json_body
    print(json_packet)

    smtp_server = "smtp.aoc.nrao.edu"
    sender_email = "do-not-reply@nrao.edu"  # Enter your address
    receiver_email = json_packet["destination_email"]  # Enter receiver address
    template_name = request.matchdict["template_name"]

    try:
        template = request.notification_info.lookup_template(template_name)
    except Exception as e:
        return Response("An error occurred looking up template: "+template_name+"\nerror: "+str(e))


    if template is None:
        return Response("No template '"+template_name+"' found.")


    args = {
        'template': template.content,
        'data': json_packet
    }
    try:
        contents = chevron.render(**args)
    except Exception as e:
        return Response("An error occurred rendering template: "+str(template)+"\nerror: "+str(e))



    with smtplib.SMTP(smtp_server) as server:
        r = server.sendmail(sender_email, receiver_email, contents)

    if r == {}:
        return Response("Email sent.\n"+contents)
    else:
        return Response("Unable to send to address(es): "+r)

def create_template(request):
    name = request.json_body["name"]
    description = request.json_body["description"]
    template = request.json_body["template"]

    ##check that we got all the necessary values:
    if len(name) <= 0:
        return Response("Unable to create template.  No value received for template name.")

    if len(description) <= 0:
        return Response("Unable to create template.  No value received for template description.")

    if len(template) <= 0:
        return Response("Unable to create template.  No value received for template text.")


    try:
        new_template = request.notification_info.create_template(name, description, template)
    except Exception as e:
        return Response("An error occurred saving template: "+name+".  \nError: "+str(e))


    return Response("Template "+name+" created.")


def delete_template(request):
    template_name = request.matchdict["template_name"]

    try:
        deleted = request.notification_info.delete_template(template_name)
    except Exception as e:
        return Response("An error occurred deleting template: "+template_name+".  \nError: "+str(e))


    return Response("Template "+template_name+" deleted.")


def list_templates(request):
    response = "Available notification templates:\n"

    for temp in request.notification_info.all_templates():
        response += temp.name+"\n"

    return Response(response)

def main(global_config, **settings):
    with Configurator(settings=settings) as config:
        ##TODO: this should probably be changed to NotificationSettings, and new settings added to Capo.
        settings = CapoConfig().settings("edu.nrao.archive.workspaces.CapabilitySettings")

        print("settings: "+settings.__str__())

        session_factory = get_session_factory(get_engine())
        config.registry["dbsession_factory"] = session_factory

        # make notification_info available for use in Pyramid
        config.add_request_method(
            # r.tm is the transaction manager used by pyramid_tm
            lambda request: NotificationInfo(get_tm_session(session_factory, request.tm)),
            "notification_info",
            reify=True,
        )

        config.add_route('notify', '/notify/')
        config.add_route('send', '/notify/{template_name}/send')
        config.add_route('create', '/notify/create/')
        config.add_route('delete', '/notify/{template_name}/delete')

        config.add_view(list_templates, route_name='notify')
        config.add_view(send_email, route_name='send')
        config.add_view(create_template, route_name='create')
        config.add_view(delete_template, route_name='delete')

        notification = config.make_wsgi_app()
        server = make_server('0.0.0.0', 3458, notification)
        server.serve_forever()

if __name__ == "__main__":
    main()